/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

import org.apache.commons.io.FilenameUtils;

import java.util.ArrayList;
import java.util.List;

public abstract class AbstractLinker implements Linker {

	public AbstractLinker() {
	}

	public AbstractLinker(AbstractLinker linker) {
		this.debugEnabled = linker.debugEnabled;
		this.runtimeLibraryUse = linker.runtimeLibraryUse;
		this.binariesDirectory = linker.binariesDirectory;
		this.objectDirectory = linker.objectDirectory;
		this.staticLibraries = linker.staticLibraries;
		this.dynamicLibraries = linker.dynamicLibraries;
		this.globalLibraryDirectories = linker.globalLibraryDirectories;
		this.libraryDirectories = linker.libraryDirectories;
	}

	public void setDebugEnabled(boolean debugEnabled) {
		this.debugEnabled = debugEnabled;
	}

	public boolean getDebugEnabled() {
		return this.debugEnabled;
	}

	public void setBinariesDirectory(String binariesDirectory) {
		this.binariesDirectory = binariesDirectory;
	}

	public String getBinariesDirectory() {
		return this.binariesDirectory;
	}

	public void setObjectDirectory(String objectDirectory) {
		this.objectDirectory = objectDirectory;
	}

	public String getObjectDirectory() {
		return this.objectDirectory;
	}

	public boolean addStaticLibraryDependence(String libraryName) {
		return staticLibraries.add(libraryName);
	}

	public boolean removeStaticLibraryDependence(String libraryName) {
		return staticLibraries.remove(libraryName);
	}

	public void clearStaticLibraryDependences() {
		staticLibraries.clear();
	}
    
    public boolean addDynamicLibraryDependence(String libraryName) {
        return dynamicLibraries.add(libraryName);
    }
    
    public boolean removeDynamicLibraryDependence(String libraryName) {
        return dynamicLibraries.remove(libraryName);
    }
    
    public void clearDynamicLibraryDependences() {
        dynamicLibraries.clear();
    }

	public void setRuntimeLibraryUse(boolean runtimeLibraryUse) {
		this.runtimeLibraryUse = runtimeLibraryUse;
	}

	public boolean getRuntimeLibraryUse() {
		return this.runtimeLibraryUse;
	}

	public boolean addGlobalLibraryDirectory(String libraryDirectory) {
		return globalLibraryDirectories.add(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public boolean removeGlobalLibraryDirectory(String libraryDirectory) {
		return globalLibraryDirectories.remove(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public void clearGlobalLibraryDirectories() {
		globalLibraryDirectories.clear();
	}

	public boolean addLibraryDirectory(String libraryDirectory) {
		return libraryDirectories.add(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public boolean removeLibraryDirectory(String libraryDirectory) {
		return libraryDirectories.remove(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public void clearLibraryDirectories() {
		libraryDirectories.clear();
	}

	public void clearAll() {
		this.runtimeLibraryUse = true;
		this.clearDynamicLibraryDependences();
		this.clearGlobalLibraryDirectories();
		this.clearLibraryDirectories();
		this.clearStaticLibraryDependences();
	}

	protected boolean debugEnabled = false;
	protected boolean runtimeLibraryUse = true;
	protected String binariesDirectory = null;
	protected String objectDirectory = null;
	protected List<String> staticLibraries = new ArrayList<String>();
    protected List<String> dynamicLibraries = new ArrayList<String>();
	protected List<String> globalLibraryDirectories = new ArrayList<String>();
	protected List<String> libraryDirectories = new ArrayList<String>();
}
