/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

import info.yeppp.Build.sdk.Windows7Sdk;
import info.yeppp.Build.sdk.Windows8Sdk;
import info.yeppp.Build.sdk.WindowsSdk;
import org.apache.commons.lang3.SystemUtils;

public class MainApp {

	public static void main(String[] args) throws Exception {
		LibraryBuilder.Toolchain toolchain = null;
		String toolchainName = SystemUtils.IS_OS_WINDOWS ? "x64-vs2010" : "x64-gcc";
		for (String arg : args) {
			if (arg.startsWith("--toolchain:")) {
				toolchainName = arg.substring("--toolchain:".length());
			}
		}
		if (toolchainName.equals("x64-vs2011")) {
			final Architecture architecture = Architecture.X64;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows8Sdk();
			final VisualStudio visualStudio = new VisualStudio2011(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.linker.setDebugEnabled(true);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("x64-vs2010")) {
			final Architecture architecture = Architecture.X64;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2010(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("x64-vs2008")) {
			final Architecture architecture = Architecture.X64;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2008(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("x86-vs2011")) {
			final Architecture architecture = Architecture.X86;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows8Sdk();
			final VisualStudio visualStudio = new VisualStudio2011(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("x86-vs2010")) {
			final Architecture architecture = Architecture.X86;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2010(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("x86-vs2008")) {
			final Architecture architecture = Architecture.X86;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2008(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("ia64-vs2010")) {
			final Architecture architecture = Architecture.IA64;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2010(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("ia64-vs2008")) {
			final Architecture architecture = Architecture.IA64;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2008(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("arm-vs2011")) {
			final Architecture architecture = Architecture.ARM;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			final WindowsSdk windowsSdk = new Windows8Sdk();
			final VisualStudio visualStudio = new VisualStudio2011(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("x64-gcc")) {
			final Architecture architecture = Architecture.X64;
			toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
			toolchain.compiler = new GnuCppCompiler();
			toolchain.linker = new GnuLinker();
			toolchain.librarian = new GnuLibrarian();
			toolchain.nasm = new NetwideAssembler();
		} else if (toolchainName.equals("arm-gcc")) {
            final Architecture architecture = Architecture.ARM;
            toolchain = new LibraryBuilder.Toolchain(architecture, toolchainName);
            toolchain.compiler = new GnuCppCompiler();
            toolchain.linker = new GnuLinker();
            toolchain.librarian = new GnuLibrarian();
            toolchain.nasm = null;
			toolchain.gas = new GnuAssembler();
        } else {
            System.err.println(String.format("Unsupported toolchain name: %s", toolchainName));
        }

		System.out.println(String.format("Building for %s", toolchain.target));

		final String projectDirectory = System.getProperty("user.dir");
		LibraryBuilder builder = new LibraryBuilder(projectDirectory, toolchain);
		builder.compileLibrary();
		builder.buildPerformanceTests();
		builder.buildExamples();
	}

}
