/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build.gui;

import java.awt.*;

class BuildItem {
	enum State {
		Waiting,
		Building,
		Built,
		Failed
	}

	private State state;
	private String name;
	private int warnings;
	private int errors;

	public BuildItem(String name) {
		this.state = State.Waiting;
		this.name = name;
		this.warnings = 0;
		this.errors = 0;
	}

	public Color getColor() {
		switch (state) {
			case Waiting:
				return Color.CYAN;
			case Building:
				return Color.YELLOW;
			case Built:
				return Color.GREEN;
			case Failed:
				return Color.RED;
			default:
				assert false;
				return null;
		}
	}

	public String getName() {
		return name;
	}

	public int getWarnings() {
		return warnings;
	}

	public int getErrors() {
		return errors;
	}
	
	public State getState() {
		return state;
	}
	
	public BuildItem setState(State state) {
		this.state = state;
		return this;
	}
}

