/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build.sdk;

import info.yeppp.Build.Version;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AndroidNdk {

	private static class Toolchain {
		public Toolchain(String path, Version version) {
			this.path = path;
			this.version = version;
		}

		final String path;
		final Version version;
	}

	public AndroidNdk() {
		this.installPath = System.getenv("ANDROID_NDK_ROOT");
		if (this.installPath == null) {
			throw new UnsupportedOperationException("Android NDK was not found on this computer");
		}
		final List<Toolchain> x86Toolchains = new LinkedList<Toolchain>();
		final List<Toolchain> armToolchains = new LinkedList<Toolchain>();
		final List<Toolchain> mipsToolchains = new LinkedList<Toolchain>();
		final Collection<File> directories = FileUtils.listFiles(new File(FilenameUtils.concat(installPath, "toolchains")), null, false);
		for (File file: directories) {
			if (file.isDirectory()) {
				final String name = file.getName();
				final Matcher matcher = toolchainPattern.matcher(name);
				if (matcher.find()) {
					final String abi = matcher.group(1);
					final Version version = new Version(matcher.group(2));
					if (abi.equals("arm")) {
						armToolchains.add(new Toolchain(file.getAbsolutePath(), version));
					} else if (abi.equals("x86")) {
						x86Toolchains.add(new Toolchain(file.getAbsolutePath(), version));
					} else if (abi.equals("mipsel")) {
						mipsToolchains.add(new Toolchain(file.getAbsolutePath(), version));
					}
				}
			}
		}
		if (armToolchains.isEmpty()) {
			armToolchainPath = null;
		} else {
			final Iterator<Toolchain> toolchainIterator = armToolchains.iterator();
			Toolchain latestToolchain = toolchainIterator.next();
			while (toolchainIterator.hasNext()) {
				final Toolchain toolchain = toolchainIterator.next();
				if (toolchain.version.compareTo(latestToolchain.version) > 0) {
					latestToolchain = toolchain;
				}
			}
			armToolchainPath = FilenameUtils.concat(latestToolchain.path, "prebuilt/windows/bin");
		}
		if (x86Toolchains.isEmpty()) {
			x86ToolchainPath = null;
		} else {
			final Iterator<Toolchain> toolchainIterator = x86Toolchains.iterator();
			Toolchain latestToolchain = toolchainIterator.next();
			while (toolchainIterator.hasNext()) {
				final Toolchain toolchain = toolchainIterator.next();
				if (toolchain.version.compareTo(latestToolchain.version) > 0) {
					latestToolchain = toolchain;
				}
			}
			x86ToolchainPath = FilenameUtils.concat(latestToolchain.path, "prebuilt/windows/bin");
		}
		if (mipsToolchains.isEmpty()) {
			mipsToolchainPath = null;
		} else {
			final Iterator<Toolchain> toolchainIterator = mipsToolchains.iterator();
			Toolchain latestToolchain = toolchainIterator.next();
			while (toolchainIterator.hasNext()) {
				final Toolchain toolchain = toolchainIterator.next();
				if (toolchain.version.compareTo(latestToolchain.version) > 0) {
					latestToolchain = toolchain;
				}
			}
			mipsToolchainPath = FilenameUtils.concat(latestToolchain.path, "prebuilt/windows/bin");
		}
	}

	private final Pattern toolchainPattern = Pattern.compile("(arm|mipsel|x86)\\-(?:\\w+\\-)*(\\d+(?:\\.\\d+)*)");
	private final String installPath;
	private final String x86ToolchainPath;
	private final String armToolchainPath;
	private final String mipsToolchainPath;
}
