/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build.sdk;

import info.yeppp.Build.internals.FileSystem;
import org.apache.commons.lang3.SystemUtils;

public class Java6Sdk extends JavaSdk {

	public Java6Sdk() {
		try {
			installPath = info.yeppp.Build.internals.WinRegistry.readString(info.yeppp.Build.internals.WinRegistry.HKEY_LOCAL_MACHINE, "Software\\JavaSoft\\Java Development Kit\\1.6", "JavaHome");
		} catch (Exception e) {
		}
		if (installPath == null) {
			throw new UnsupportedOperationException("Java 1.6 SDK was not found on this system and can not be used");
		}
		toolPath = FileSystem.lookupExistingDirectory(installPath, "bin");
		if (toolPath == null) {
			throw new UnsupportedOperationException("Java 1.6 SDK bin directory was not found on disk");
		}
		jniIncludePath = FileSystem.lookupExistingDirectory(installPath, "include");
		if (jniIncludePath == null) {
			throw new UnsupportedOperationException("Java 1.6 SDK include directory was not found on disk");
		}
		jniPlatformIncludePath = FileSystem.lookupExistingDirectory(jniIncludePath, "win32");
		if (SystemUtils.IS_OS_WINDOWS) {
			if (jniPlatformIncludePath == null) {
				throw new UnsupportedOperationException("Java 1.6 SDK win32-specific include directory was not found on disk");
			}
		}
	}

	public String getName() {
		return "java6";
	}
}
